

import java.awt.EventQueue;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JToolBar;
import java.awt.BorderLayout;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import org.freepascal.rtl.TObject;


import SecureBlackbox.Base.*;
import SecureBlackbox.MIME.SBMIME;
import SecureBlackbox.MIME.TElMessageHeader;
import SecureBlackbox.MIME.TElMessageHeaderField;
import SecureBlackbox.Mail.*;
import SecureBlackbox.SSLCommon.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.URL;

public class Main extends JDialog {

	private static final long serialVersionUID = 1L;
	private JTable table;
	private TElPOP3Client client;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main dialog = new Main();
					dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
					dialog.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the dialog.
	 */
	public Main() {
		setTitle("EldoS POP3 client demo");
		setBounds(100, 100, 483, 319);
		
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		
		JMenu mnFile = new JMenu("File");
		menuBar.add(mnFile);
		
		JMenuItem mntmConnect = new JMenuItem("Connect");
		mntmConnect.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				try {
					
					Connect();
					
				} catch (EElPOP3Error ex) {
					showErrorMessage(ex.getMessage() + " [ErrorCode = " + ex.getErrorCode() + "]", "Error");
					client.close(true);
				}
			}
		});
		mnFile.add(mntmConnect);
		
		JMenuItem mntmDisconnect = new JMenuItem("Disconnect");
		mntmDisconnect.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				Disconnect();
			}
		});
		mnFile.add(mntmDisconnect);
		
		JMenuItem mntmExit = new JMenuItem("Exit");
		mntmExit.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				dispose();
				System.exit(0);
			}
		});
		mnFile.add(mntmExit);
		
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		getContentPane().add(toolBar, BorderLayout.NORTH);
		
		JButton btnConnect = new JButton("");
		btnConnect.setIcon(getIcon("connect"));
		btnConnect.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					
					Connect();
					
				} catch (EElPOP3Error ex) {
					showErrorMessage(ex.getMessage() + " [ErrorCode = " + ex.getErrorCode() + "]", "Error");
					client.close(true);
				}
			}
		});
		toolBar.add(btnConnect);
		
		JButton btnDisconnect = new JButton("");
		btnDisconnect.setIcon(getIcon("disc"));
		btnDisconnect.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Disconnect();
			}
		});
		toolBar.add(btnDisconnect);
		
		JButton btnDownload = new JButton("");
		btnDownload.setIcon(getIcon("down"));
		btnDownload.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					Download();
				} catch (EElPOP3Error ex) {
					showErrorMessage(ex.getMessage() + " [ErrorCode = " + ex.getErrorCode() + "]", "Error");
				}
			}
		});
		toolBar.add(btnDownload);
		
		JButton btnDelete = new JButton("");
		btnDelete.setIcon(getIcon("close"));
		btnDelete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					Delete();
				} catch (EElPOP3Error ex) {
					ex.printStackTrace();
					showErrorMessage(ex.getMessage() + " [ErrorCode = " + ex.getErrorCode() + "]", "Error");
				}
			}
		});
		toolBar.add(btnDelete);
		
		JScrollPane scrollPane = new JScrollPane();
		getContentPane().add(scrollPane, BorderLayout.CENTER);
		
		table = new JTable(new ConTableModel());
		table.setFillsViewportHeight(true);
		scrollPane.setViewportView(table);

		init();
	}

	private Icon getIcon(String name) {
		URL resPath = getClass().getResource(name + ".ico");
		if (resPath == null) return new ImageIcon();
		return new ImageIcon(resPath);
	}
	
	private void init() {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");
		
		client = new TElPOP3Client();
		client.setOnCertificateValidate(new TSBCertificateValidateEvent(client_OnCertificateValidate));		
	}

	TSBCertificateValidateEvent.Callback client_OnCertificateValidate = new TSBCertificateValidateEvent.Callback() {
		
		public void tsbCertificateValidateEventCallback(TObject sender,
				TElX509Certificate cert, TSBBoolean accept) {
			accept.value = true;
		}
	};
	
	private void showErrorMessage(String msg, String cap) {
		JOptionPane.showMessageDialog(null, msg, cap, JOptionPane.ERROR_MESSAGE);		
	}
	
	
	private void Connect() throws EElPOP3Error
	{
		if (client.getActive())
		{
			showErrorMessage("You are already connected! Please disconnect first.", "Error");
			return;
		}

		ConnectDialog Frm = new ConnectDialog();
		Frm.setModal(true);
		Frm.setVisible(true);
		
		if (Frm.isOK())
		{
			client.setAddress(Frm.getServer());
			client.setPort(Frm.getPort());
			client.setUseSSL(Frm.getUseSSL());						

			switch (Frm.getSSLMode())
			{
				case 0: 
					client.setSSLMode(TSBSSLMode.smExplicit);
					break;
				case 1:
					client.setSSLMode(TSBSSLMode.smImplicit);
					break;
			}

			try
			{
				client.open();
				client.setUsername(Frm.getUsername());
				client.setPassword(Frm.getPassword());
				client.login();
				TElPOP3MailboxInfo info = new TElPOP3MailboxInfo();
				client.getMailboxInfo(info);
				
				if (chooseMessage("You have " + info.Messages + " messages in the Inbox.\n" +
					"Total messages size is " + info.TotalSize + " bytes.\n" +
					"Do you want to upload the headers?"))
				{
					GetMessageList();
				}
			}
			catch (Exception e)
			{
				e.printStackTrace();
				showErrorMessage(e.getMessage(), "Error");
				client.close(true);
			}
		}
	}
	
	private boolean chooseMessage(String msg) {
		return (JOptionPane.showConfirmDialog(this, msg) == JOptionPane.YES_OPTION);
	}


	private void Disconnect()
	{
		if (client.getActive())
			client.close(false);
	}

	private void Download() throws EElPOP3Error
	{
		String fileName = getSaveFileName();
		if ((table.getSelectedRows().length == 1) && (fileName.length() > 0))
		{
			TElFileStream FS = new TElFileStream(fileName, "rw", true);
			try
			{
				TSBPOP3MessageID ID = ((ConTableModel)table.getModel()).getTag(table.getSelectedRow());
				client.receiveMessage(ID.Index, FS);
			}
			finally
			{
				FS.Free();
			}
		}
	}

	String getSaveFileName(){
		JFileChooser fc = new JFileChooser();

	    int returnVal = fc.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}

	private void Delete() throws EElPOP3Error
	{
		int[] indexes = table.getSelectedRows();
		for (int i = 0; i < indexes.length; i++)
		{
			TSBPOP3MessageID ID = ((ConTableModel)table.getModel()).getTag(indexes[i]);
			client.deleteMessage(ID.Index);
		}
		GetMessageList();
	}
	
	private void GetMessageList()
	{
		TElPOP3MessageIDListRef IDs = new TElPOP3MessageIDListRef();
		client.getMessageIDList(IDs);
		TSBObject h = new TSBObject();
		TSBString charset = new TSBString();
		long size = 0;

		ConTableModel model = ((ConTableModel)table.getModel());
		try
		{
			model.clear();

			if (IDs.Value != null)
			{
				for (int i = 0; i < IDs.Value.length; i++)
				{
					if (client.receiveMessageHeader(IDs.Value[i].Index, charset, false, h) == SBMIME.EL_OK)
					{
						size = client.getMessageSize(IDs.Value[i].Index);
	
						TElMessageHeader header = (TElMessageHeader) h.value;
						
						TElMessageHeaderField field = header.getField("From", 0);
						String f = (field != null) ? field.getValue() : "";
						
						field = header.getField("Subject", 0);
						String s = (field != null) ? field.getValue() : "";
						
						String z = size + "";
						TSBPOP3MessageID tag = IDs.Value[i];
	
						model.addRow(f, s, z, tag);
					}
				}
			}
		}
		finally
		{
		}
	}	
}
