import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPasswordField;
import javax.swing.JComboBox;
import javax.swing.ButtonGroup;

import SecureBlackbox.Base.TElWinCertStorage;
import SecureBlackbox.Base.TElX509Certificate;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class SelCertForm extends JDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	private JTextField tbCert;
	private JPasswordField tbCertPassword;
	private final ButtonGroup buttonGroup = new ButtonGroup();
	private JRadioButton rbUseCertFile;
	private JRadioButton rbUseSystemCert;
	private JComboBox<String> cbSystemCerts;
	private boolean isOk;
	private TElWinCertStorage SystemStore;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			SelCertForm dialog = new SelCertForm();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public SelCertForm() {
		setTitle("Select Certificate");
		setBounds(100, 100, 450, 300);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		
		rbUseCertFile = new JRadioButton("Use certificate from file");
		buttonGroup.add(rbUseCertFile);
		rbUseCertFile.setSelected(true);
		rbUseCertFile.setBounds(6, 18, 237, 23);
		contentPanel.add(rbUseCertFile);
		
		tbCert = new JTextField();
		tbCert.setBounds(30, 48, 306, 20);
		contentPanel.add(tbCert);
		tbCert.setColumns(10);
		
		JButton btnBrowse = new JButton("Browse...");
		btnBrowse.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				browseClick();
			}
		});
		btnBrowse.setBounds(346, 47, 91, 23);
		contentPanel.add(btnBrowse);
		
		JLabel lblCertificatePassword = new JLabel("Certificate password");
		lblCertificatePassword.setBounds(30, 79, 212, 14);
		contentPanel.add(lblCertificatePassword);
		
		tbCertPassword = new JPasswordField();
		tbCertPassword.setBounds(30, 97, 143, 20);
		contentPanel.add(tbCertPassword);
		
		rbUseSystemCert = new JRadioButton("Use certificate from system certificate store");
		buttonGroup.add(rbUseSystemCert);
		rbUseSystemCert.setBounds(6, 140, 357, 23);
		contentPanel.add(rbUseSystemCert);
		
		cbSystemCerts = new JComboBox<String>();
		cbSystemCerts.setBounds(40, 170, 323, 23);
		contentPanel.add(cbSystemCerts);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						isOk = true;
						setVisible(false);
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						if (SystemStore != null)
							SystemStore.Free();
						dispose();
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
		
		init();
	}

	private void init() {
		SystemStore = new TElWinCertStorage();
		RefreshSystemCertificateList();
	}

	protected void browseClick() {
		String fileName = getOpenFileName();
		if (fileName.length() > 0)
			tbCert.setText(fileName);	
	}
	
	String getOpenFileName(){
		JFileChooser fc = new JFileChooser();
	    	    
		int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
	public TElX509Certificate getCert() throws Exception {
		TElX509Certificate Cert = null;
		// loading certificate
		if (rbUseCertFile.isSelected()) 
		{
			Cert = new TElX509Certificate();
			try
			{
				Cert.loadFromFileAuto(tbCert.getText(), new String(tbCertPassword.getPassword()));
			}
			catch(Exception ex)
			{
				Cert.Free();
				Cert = null;
				throw ex;
			}
		} 
		else 
		{
			TElX509Certificate TempCert = SystemStore.getCertificate(cbSystemCerts.getSelectedIndex());
			if (TempCert != null)
			{
				Cert = new TElX509Certificate();
				TempCert.clone(Cert, true);
			}
		}

		return Cert;
	}
	
	public void RefreshSystemCertificateList() 
	{
		cbSystemCerts.removeAllItems();
		try 
		{
			SystemStore.getSystemStores().clear();
			SystemStore.getSystemStores().add("MY");
		} 
		finally 
		{
		}
		for (int i = 0; i < SystemStore.getCount(); i++) 
		{
			cbSystemCerts.addItem(SystemStore.getCertificate(i).getSubjectName().CommonName);
		}
	}
	
	void showErrorMessage(String msg){
		JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);
	}

	public boolean isOK() {
		return isOk;
	}
}
