import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import SecureBlackbox.PDF.TElPDFPublicKeyRevocationInfo;


public class RevTableModel extends AbstractTableModel {
	private static final long serialVersionUID = 2771284071470299948L;
	public static final int TYPE_INDEX = 0;
	public static final int DTLS_INDEX = 1;
	public static final int LOCT_INDEX = 2;
	
	protected Vector<RevDataRow> dataVector = new Vector<RevDataRow>();
	
    protected String[] columnNames = {
    		"Type", 
    		"Details",
    		"Location"
    };
    
    public void addRow(TElPDFPublicKeyRevocationInfo revInfo, String location) {
    	
        for (int i = 0; i < revInfo.getCertificates().getCount(); i++)
        {
        	RevDataRow row = new RevDataRow();
            row.Type = "Certificate";
            row.Details = revInfo.getCertificates().getCertificate(i).getSubjectRDN().saveToDNString();
            row.Location = location;
            row.Tag = revInfo.getCertificates().getCertificate(i);
        	dataVector.addElement(row);
        	fireTableDataChanged();        	
        }
        for (int i = 0; i < revInfo.getCRLCount(); i++)
        {
        	RevDataRow row = new RevDataRow();
            row.Type = "CLR";
            row.Details = "Issuer: " + revInfo.getCRL(i).getIssuer().saveToDNString() + ", time: " + revInfo.getCRL(i).getThisUpdate().toString();
            row.Location = location;
            row.Tag = revInfo.getCRL(i);
        	dataVector.addElement(row);
        	fireTableDataChanged();        	
        }
        for (int i = 0; i < revInfo.getOCSPResponseCount(); i++)
        {
        	RevDataRow row = new RevDataRow();
            row.Type = "OCSP response";
            row.Details = "No details";
            row.Location = location;
            row.Tag = revInfo.getCRL(i);
        	dataVector.addElement(row);
        	fireTableDataChanged();
        }    	
    }
    
	public String getColumnName(int column) {
        return columnNames[column];
    }
    
	public int getColumnCount() {
		return columnNames.length;
	}

	public int getRowCount() {
		return dataVector.size();
	}

    public Object getValueAt(int row, int col) {
        RevDataRow record = (RevDataRow)dataVector.get(row);
        switch (col) {
        	case TYPE_INDEX:
        		return record.Type;
        	case DTLS_INDEX:
	        	return record.Details;
        	case LOCT_INDEX:
        		return record.Location; 
            default:
               return new Object();
        }
    }

	public void clear() {
		dataVector.removeAllElements();
		fireTableDataChanged();
	}

	public Object getTag(int i) {
		return dataVector.get(i).Tag;
	}
}