import javax.crypto.*;
import javax.crypto.spec.*;
import java.security.*;
import SecureBlackbox.jce.provider.*;
import SecureBlackbox.jce.interfaces.*;
import SecureBlackbox.jce.spec.*;

public class JCEProviderSample
{
    // This sample illustrates how to use JCE cryptoprovider to encrypt/decrypt a data
    // using AES-128-CFB symmetric cipher

    public static void main(String[] args)
    throws java.lang.Exception 
    {
        #error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
        final String licenseKey = "<put the key here>";
        final String input = "123456789012345678901234567890";
        
        // register cryptoprovider
        Provider provider = new SecureBlackboxProvider(licenseKey);
        Security.addProvider(provider);
        
        // generate random 128-bit key
        KeyGenerator kgen = KeyGenerator.getInstance("AES", "SecureBlackbox");
        kgen.init(128);
        SecretKey skey = kgen.generateKey();
        SecretKeySpec skeySpec = new SecretKeySpec(skey.getEncoded(), "AES");
        
        // generate 16-bytes long initialization vector
        SecureRandom r = SecureRandom.getInstance("SHA1PRNG", "SecureBlackbox");
        byte[] buf = new byte[16];
        r.nextBytes(buf);
        IvParameterSpec iv = new IvParameterSpec(buf); 
       
        // encrypt input text
        Cipher cipher = Cipher.getInstance("AES/CFB/PKCS5", "SecureBlackbox");
        cipher.init(Cipher.ENCRYPT_MODE, skeySpec, iv);
        byte[] output = cipher.doFinal(input.getBytes());
        
        // decrypt ciphertext
        cipher = Cipher.getInstance("AES/CFB/PKCS5", "SecureBlackbox");
        cipher.init(Cipher.DECRYPT_MODE, skeySpec, iv);
        byte[] output2 = cipher.doFinal(output);
        String outputStr = new String(output2);
        
        // check if decrypted string is the same as input one
        if ( ! outputStr.equals(input))
          throw new Exception("AES test failed!");
          
        System.out.println("AES test succeed!");
    }
}
