import java.util.*;

import javax.swing.table.AbstractTableModel;

public class ConTableModel extends AbstractTableModel {
	private static final long serialVersionUID = 1L;
	public static final int ICON_INDEX = 0;
	public static final int HOST_INDEX = 1;
	public static final int R_INDEX = 2;
	public static final int S_INDEX = 3;
	public static final int I_INDEX = 4;
	public static final int O_INDEX = 5;

	protected Vector<ConDataRow> dataVector = new Vector<ConDataRow>();
	
    protected String[] columnNames = {
    		"",
    		"Host", 
    		"Received",
    		"Sent",
    		"Input state",
    		"Output state"
    };

	public int getColumnCount() {
		return columnNames.length;
	}

 	public String getColumnName(int column) {
        return columnNames[column];
    }

 	public Class<?> getColumnClass(int column) {
       	return String.class;
    }

	public int getRowCount() {
		return dataVector.size();
	}

    public void setValueAt(Object value, int row, int column) {
      	 ConDataRow record = (ConDataRow)dataVector.get(row);
           switch (column) {
           	   case ICON_INDEX:
           		   record.Status = (String)value;
           		   break;
               case HOST_INDEX:
                  record.Host = (String)value;
                  break;
               case R_INDEX:
                   record.Received = (String)value;
                   break;
               case S_INDEX:
                   record.Sent = (String)value;
                   break;
               case I_INDEX:
                   record.InState = (String)value;
                   break;
               case O_INDEX:
                   record.OutState = (String)value;
                   break;
               default:
                  System.out.println("invalid index");
           }
           fireTableCellUpdated(row, column);
       } 	
	
	public Object getValueAt(int row, int col) {
        ConDataRow record = (ConDataRow)dataVector.get(row);
        switch (col) {
        	case ICON_INDEX:
        		return record.Status;
	        case HOST_INDEX:
	        	return record.Host;
            case R_INDEX:
            	return record.Received;
            case S_INDEX:
            	return record.Sent;
            case I_INDEX:
            	return record.InState;
            case O_INDEX:
            	return record.OutState;                
            default:
            	return new Object();
        }
	}

	public void addRow(SshForwarding c) {
		String host = c.getToHost();
		String received = c.getReceived() + "";
		String sent = c.getSent() + "";

		String instate = c.getInState().toString();
		String outstate = c.getOutState().toString();
		
		String status = c.getCurrentStatusState();
	
		Object tag = c;
		dataVector.addElement(new ConDataRow(status, host, received, sent, instate, outstate, tag));
		fireTableDataChanged();
	}

	public void change(SshForwarding c) {
		remove(c);
		addRow(c);
	}

	public void remove(SshForwarding c) {
		int index = -1;
		for (int i = 0; i < dataVector.size(); i++) 
		{
			if (dataVector.get(i).Tag.equals(c)) 
			{
				index = i;
				break;
			}
		}
		
		if (index != -1)
			dataVector.remove(index);
		
		fireTableDataChanged();
	}
}
