import java.io.FileInputStream;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.swing.JPasswordField;
import javax.swing.table.AbstractTableModel;

import SecureBlackbox.Base.SBConstants;
import SecureBlackbox.Base.SBUtils;
import SecureBlackbox.Base.TElCustomCertStorage;
import SecureBlackbox.Base.TElMemoryCertStorage;
import SecureBlackbox.Base.TElRelativeDistinguishedName;
import SecureBlackbox.Base.TElX509Certificate;

public class OptionsTableModel extends AbstractTableModel {

	private static final long serialVersionUID = 1L;
	public static final int S_INDEX = 0;
	public static final int I_INDEX = 1;
    public static final int A_INDEX = 2; 
    
    protected String[] columnNames = {"Subject", "Issuer", "Algorithm" };
    protected Vector<OptionsDataRow> dataVector = new Vector<OptionsDataRow>();
    
    public OptionsTableModel(String certFileName) throws Exception{
    	int r;
    	int FilterIndex = getFilterIndex(certFileName);
    	FileInputStream f = new FileInputStream(certFileName);

		TElX509Certificate cert = new TElX509Certificate();
		if (FilterIndex == 1) 
		{
			r = cert.loadFromStreamPFX(f, requestPassword(), 0);
		} 
		else if (FilterIndex == 2) 
		{
			r = cert.loadFromStreamPEM(f, requestPassword(), 0);
		} 
		else if (FilterIndex == 3) 
		{
			try 
			{
				cert.loadFromStream(f, 0);
				r = 0;
			} 
			catch (Exception ex)
			{
				r = -1;
			}
		}
		else 
		{
			r = -1;
		}
		if (r != 0) 
		{
			showMessage("", "Cannot load certificate, error " + r);
		} 
		else 
		{
			String s = FormatRDN(cert.getSubjectRDN());
			String i = FormatRDN(cert.getIssuerRDN());
			String a = "";
			dataVector.add(new OptionsDataRow(s, i, a, cert));
		}
		 
		f.close();
	}    	
	
 	public String getColumnName(int column) {
        return columnNames[column];
    }
 	
    static void showMessage(String caption, String msg) {
		JOptionPane.showMessageDialog(null, msg, caption, JOptionPane.INFORMATION_MESSAGE);
	}

	private String FormatRDN(TElRelativeDistinguishedName rdn) 
	{
		String s = "";
		int i;
		for (i = 0; i < rdn.getCount(); i++) 
		{
			if (SBUtils.compareMem(SBConstants.SB_CERT_OID_COMMON_NAME.Data, rdn.getOID(i))) 
			{
				s = s + "CN=" + SBUtils.stringOfBytes(rdn.getValue(i)) + "; ";
			} 
			else if (SBUtils.compareMem(SBConstants.SB_CERT_OID_EMAIL.Data, rdn.getOID(i)))
			{
				s = s + "E=" + SBUtils.stringOfBytes(rdn.getValue(i)) + "; ";
			}
			else if (SBUtils.compareMem(SBConstants.SB_CERT_OID_COUNTRY.Data, rdn.getOID(i)))
			{
				s = s + "C=" + SBUtils.stringOfBytes(rdn.getValue(i)) + "; ";
			}
			else if (SBUtils.compareMem(SBConstants.SB_CERT_OID_ORGANIZATION.Data, rdn.getOID(i)))
			{
				s = s + "O=" + SBUtils.stringOfBytes(rdn.getValue(i)) + "; ";
			}
			else if (SBUtils.compareMem(SBConstants.SB_CERT_OID_STATE_OR_PROVINCE.Data, rdn.getOID(i)))
			{
				s = s + "SP=" + SBUtils.stringOfBytes(rdn.getValue(i)) + "; ";
			}
		}
		return s;
	}
	
    private String requestPassword() {
		JPasswordField jpf = new JPasswordField();
		int result = JOptionPane.showConfirmDialog(null, jpf, "Please enter password", JOptionPane.OK_CANCEL_OPTION);
		if (result == 0)
			return new String(jpf.getPassword());
		return "";
	}
	
	private int getFilterIndex(String certFileName) {
		String ext = extension(certFileName).toLowerCase();
		if (ext.compareTo("pfx") == 0) return 1;
		if (ext.compareTo("pem") == 0) return 2;
		if (ext.compareTo("cer") == 0) return 3;
		return -1;
	}
    
	public String extension(String fullPath) {
	    int dot = fullPath.lastIndexOf(".");
	    return fullPath.substring(dot + 1);
	}
	
	public int getColumnCount() {
		return columnNames.length;
	}

	public int getRowCount() {
		return dataVector.size();
	}

	public Object getValueAt(int row, int col) {
		OptionsDataRow record = (OptionsDataRow)dataVector.get(row); 
		switch (col) {
	        case S_INDEX:
	       	 return record.subject;
	        case I_INDEX:
	           return record.issuer;
	        case A_INDEX:
	           return record.alg;
	        default:
	           return new Object();
		}
	}
	
	public Class<?> getColumnClass(int column) {
        switch (column) {
        	 case S_INDEX:
        	 case I_INDEX:
        	 case A_INDEX:
        		 return String.class;
            default:
               return Object.class;
        }
	}
	
	public void removeSelectedItems(int[] indexes) {
		Vector<OptionsDataRow> result = new Vector<OptionsDataRow>();
		for(int i = 0; i < dataVector.size(); i++){
			if (selectedIndex(i, indexes))
				fireTableRowsDeleted(i, i);
			else{
				OptionsDataRow row = (OptionsDataRow)dataVector.get(i);
				result.add(row);				
			}
		}
		dataVector.clear();
		dataVector.addAll(result);
	}

	private boolean selectedIndex(int i, int[] indexes) {
		for (int j = 0; j < indexes.length; j++)
			if (i == indexes[j]) return true;
		return false;
	}

	public void setCertificates(TElMemoryCertStorage str) {
		for (int k = 0; k < str.getCount(); k++) 
		{
			TElX509Certificate cert = str.getCertificate(k);
			String s = FormatRDN(cert.getSubjectRDN());
			String i = FormatRDN(cert.getIssuerRDN());
			String a = "";
			dataVector.add(new OptionsDataRow(s, i, a, cert));

		}
	}

	public TElCustomCertStorage getCertificates() {
		TElCustomCertStorage result = new TElMemoryCertStorage();
		
		for (int i = 0; i < dataVector.size(); i++)
			result.add(dataVector.get(i).tag, true);
		
		return result;
	} 
}
