import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.JLabel;

import SecureBlackbox.Base.TElWinCertStorage;
import SecureBlackbox.Base.TElX509Certificate;
import SecureBlackbox.PGP.TElPGPSecretKey;
import SecureBlackbox.XMLSecurity.SBXMLSec;
import SecureBlackbox.XMLSecurity.TElXMLKeyInfoData;
import SecureBlackbox.XMLSecurity.TElXMLKeyInfoHMACData;
import SecureBlackbox.XMLSecurity.TElXMLKeyInfoPGPData;
import SecureBlackbox.XMLSecurity.TElXMLKeyInfoRSAData;
import SecureBlackbox.XMLSecurity.TElXMLKeyInfoX509Data;
import SecureBlackbox.XMLSecurity.TElXMLVerifier;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.FileInputStream;
import javax.swing.JComboBox;


public class SelectKeyFrm extends JDialog {

	private final JPanel contentPanel = new JPanel();
	private JTextField edKeyFile;
	private JTextField edPassphrase;
	private JLabel lblMethod, lblKeyName;
	private JRadioButton rbKeyFile;
	private JComboBox cbCerts;
	private JRadioButton rbWinStore;
	
	private boolean isOk;
	
	private short signatureMethodType;
	private TElWinCertStorage certStore = new TElWinCertStorage();

	/**
	 * Create the dialog.
	 */
	public SelectKeyFrm() {
		setTitle("Select Key");
		setBounds(100, 100, 369, 343);
		getContentPane().setLayout(null);
		contentPanel.setBounds(0, 0, 434, 1);
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel);
		contentPanel.setLayout(null);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setBounds(0, 270, 350, 33);
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						isOk = true;
						setVisible(false);
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						isOk = false;
						setVisible(false);
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
		
		rbKeyFile = new JRadioButton("Use key/certificate from file:");
		rbKeyFile.setBounds(10, 79, 222, 23);
		getContentPane().add(rbKeyFile);
		
		edKeyFile = new JTextField();
		edKeyFile.setBounds(10, 109, 295, 20);
		getContentPane().add(edKeyFile);
		edKeyFile.setColumns(10);
		
		JButton btnNewButton = new JButton("...");
		btnNewButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				doSelectFile();
			}
		});
		btnNewButton.setBounds(314, 108, 26, 23);
		getContentPane().add(btnNewButton);
		
		JLabel lblNewLabel = new JLabel("Key/certificate password:");
		lblNewLabel.setBounds(10, 140, 295, 14);
		getContentPane().add(lblNewLabel);
		
		edPassphrase = new JTextField();
		edPassphrase.setBounds(10, 165, 330, 20);
		getContentPane().add(edPassphrase);
		edPassphrase.setColumns(10);
		
		lblMethod = new JLabel("Please select a key for * method");
		lblMethod.setBounds(10, 12, 330, 14);
		getContentPane().add(lblMethod);
		
		lblKeyName = new JLabel("New label");
		lblKeyName.setBounds(10, 37, 330, 14);
		getContentPane().add(lblKeyName);
		
		rbWinStore = new JRadioButton("Use Certificate from Windows store:");
		rbWinStore.setBounds(10, 202, 295, 23);
		getContentPane().add(rbWinStore);
		
		cbCerts = new JComboBox();
		cbCerts.setBounds(10, 232, 330, 20);
		getContentPane().add(cbCerts);
		
		rbKeyFile.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				rbWinStore.setSelected( ! rbKeyFile.isSelected());
			}
		});
		rbWinStore.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				rbKeyFile.setSelected( ! rbWinStore.isSelected());
			}
		});
		
		// ----------------------
		certStore.getSystemStores().clear();
		certStore.getSystemStores().add("MY");
		
		for (int i = 0; i < certStore.getCount(); i++) {
			cbCerts.addItem(certStore.getCertificate(i).getSubjectName().CommonName);
		}
	}
	
	public void init(TElXMLVerifier verifier) {
		signatureMethodType = verifier.getSignatureMethodType();
		String s;
		
		if (signatureMethodType == SBXMLSec.xmtSig)
			s = Utils.signatureMethodToString(verifier.getSignatureMethod());
		else
			s = Utils.macMethodToString(verifier.getMACMethod());
		
		lblMethod.setText(String.format("Please select a key for %s method", s));
		String keyName = verifier.getKeyName();
		lblKeyName.setText("Key Name: " + ((keyName.length() > 0) ? keyName : "none"));
	}
	
	public TElX509Certificate getSigningCertificate() {
		if (signatureMethodType != SBXMLSec.xmtSig)
			return null;
		
		if (rbKeyFile.isSelected()) {
			TElX509Certificate cert = new TElX509Certificate();
			int k = cert.loadFromFileAuto(getKeyFile(), getPassphrase());
			if (k == 0)
				return cert;
			else
				showMessage("Failed to load certificate. Error code: " + k);
		} else if (rbWinStore.isSelected()) {
			return certStore.getCertificate(cbCerts.getSelectedIndex());
		}
		
		return null;
	}
	
	public TElXMLKeyInfoData getKeyData() {
		if (signatureMethodType == SBXMLSec.xmtMAC) {
			TElXMLKeyInfoHMACData hmacKeyData = new TElXMLKeyInfoHMACData(true);
			hmacKeyData.getKey().setKey(Utils.getFileContents(getKeyFile()));
			return hmacKeyData;
		} else {
			TElX509Certificate cert = getSigningCertificate();
			if (cert != null) {
				if ( ! cert.getPrivateKeyExists())
					throw new RuntimeException("The selected certificate doesn''t contain a private key.");
				
				TElXMLKeyInfoX509Data x509KeyData = new TElXMLKeyInfoX509Data();
				x509KeyData.setCertificate(cert);
				
				return x509KeyData;
			}
			
			// trying to load file as RSA key material
			TElXMLKeyInfoRSAData rsaKeyData = new TElXMLKeyInfoRSAData(true);
			try {
				FileInputStream fs = new FileInputStream(getKeyFile());	
				try {
					rsaKeyData.getRSAKeyMaterial().setPassphrase(getPassphrase());
					rsaKeyData.getRSAKeyMaterial().loadSecret(fs, 0);
				} finally {
					fs.close();
				}
			} catch (Exception ex) {
				;
			}
			
			if (rsaKeyData.getRSAKeyMaterial().getSecretKey())
				return rsaKeyData;
			
			TElXMLKeyInfoPGPData pgpKeyData = new TElXMLKeyInfoPGPData(true);
			try {
				pgpKeyData.setSecretKey(new TElPGPSecretKey());
				pgpKeyData.getSecretKey().setPassphrase(getPassphrase());
				((TElPGPSecretKey)pgpKeyData.getSecretKey()).loadFromFile(getKeyFile());
			} catch (Exception ex) {
				;
			}
			
			if (pgpKeyData.getSecretKey() != null)
				return pgpKeyData;
			
			throw new RuntimeException("Failed to load a Key/Certificate.");
		}
	}
	
	public String getKeyFile() {
		return edKeyFile.getText();
	}
	
	public String getPassphrase() {
		return edPassphrase.getText();
	}
	
	public boolean isOK() {
		return isOk;
	}
	
	private void doSelectFile() {
		edKeyFile.setText(getFileName());
	}
	
	private String getFileName() {
		JFileChooser fc = new JFileChooser();

	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
	private void showMessage(String msg) {
    	JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.INFORMATION_MESSAGE);
	}
}
