import java.io.RandomAccessFile;
import java.util.Arrays;

import SecureBlackbox.Base.SBUtils;
import SecureBlackbox.Base.SBX509;
import SecureBlackbox.Base.TElMemoryStream;
import SecureBlackbox.Base.TElRelativeDistinguishedName;
import SecureBlackbox.Base.TSBCertificateValidity;
import SecureBlackbox.HTTPClient.TElHTTPSClient;
import SecureBlackbox.XMLSecurity.SBXMLAdES;
import SecureBlackbox.XMLSecurity.SBXMLAdESIntf;
import SecureBlackbox.XMLSecurity.SBXMLSec;


public class Utils {

	public static byte[] downloadData(String url) {
		try {
			TElHTTPSClient cli = new TElHTTPSClient();
			TElMemoryStream ms = new TElMemoryStream();
			
			cli.setOutputStream(ms);
			
			if (cli.get(url) != 200)
				throw new RuntimeException("Failed to retrieve data from: " + url);
			
			return ms.getBuffer();
		} catch (Exception ex) {
			throw new RuntimeException("Failed to retrieve data from: " + url);
		}
	}
	
	public static byte[] getFileContents(String path) {
		try {
			RandomAccessFile f = new RandomAccessFile(path, "r");
			try {
				byte[] res = new byte[(int)f.length()];
				f.read(res);
				
				return res;
			} finally {
				f.close();
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			return null;
		}
	}
	
	public static String signatureMethodToString(short method) {
		String res;
		
		switch (method) {
		case SBXMLSec.xsmDSS:
			res = "DSA-SHA1";
			break;
		case SBXMLSec.xsmRSA_SHA1:
			res = "RSA-SHA1";
			break;
		case SBXMLSec.xsmRSA_SHA256:
			res = "RSA-SHA256";
			break;
		case SBXMLSec.xsmRSA_SHA384:
			res = "RSA-SHA384";
			break;
		case SBXMLSec.xsmRSA_SHA512:
			res = "RSA-SHA512";
			break;
		case SBXMLSec.xsmRSA_MD5:
			res = "RSA-MD5";
			break;
		case SBXMLSec.xsmRSA_RIPEMD160:
			res = "RSA-RIPEMD160";
			break;
		case SBXMLSec.xsmGOST1994:
			res = "GOST R 34.10-94";
			break;
		case SBXMLSec.xsmGOST2001:
			res = "GOST R 34.10-2001";
			break;
		default:
			res = "Unknown";
		}
		
		return res;
	}
	
	public static String macMethodToString(short method) {
		String res;
		
		switch (method) {
		case SBXMLSec.xmmHMAC_MD5:
			res = "HMAC-MD5";
			break;
		case SBXMLSec.xmmHMAC_SHA1:
			res = "HMAC-SHA1";
			break;
		case SBXMLSec.xmmHMAC_SHA224:
			res = "HMAC-SHA224";
			break;
		case SBXMLSec.xmmHMAC_SHA256:
			res = "HMAC-SHA256";
			break;
		case SBXMLSec.xmmHMAC_SHA384:
			res = "HMAC-SHA384";
			break;
		case SBXMLSec.xmmHMAC_SHA512:
			res = "HMAC-SHA512";
			break;
		case SBXMLSec.xmmHMAC_RIPEMD160:
			res = "HMAC-RIPEMD160";
			break;
		default:
			res = "Unknown";
		}
		
		return res;
	}
	
	public static String x509ValidityToString(TSBCertificateValidity validity) {
		if (validity == TSBCertificateValidity.cvOk)
			return "OK";
		else if (validity == TSBCertificateValidity.cvSelfSigned)
			return "Self-signed";
		else if (validity == TSBCertificateValidity.cvInvalid)
			return "Invalid";
		else if (validity == TSBCertificateValidity.cvStorageError)
			return "Storage error";
		else if (validity == TSBCertificateValidity.cvChainUnvalidated)
			return "Chain unvalidated";
		else
			return "Unknown";
	}
	
	public static String x509ReasonToString(int reason) {
		StringBuilder sb = new StringBuilder();
		
		if ((reason & SBX509.vrBadData) == SBX509.vrBadData)
			sb.append("Bad data, ");
		if ((reason & SBX509.vrRevoked) == SBX509.vrRevoked)
			sb.append("Revoked, ");
		if ((reason & SBX509.vrNotYetValid) == SBX509.vrNotYetValid)
			sb.append("Not yet valid, ");
		if ((reason & SBX509.vrExpired) == SBX509.vrExpired)
			sb.append("Expired, ");
		if ((reason & SBX509.vrInvalidSignature) == SBX509.vrInvalidSignature)
			sb.append("Invalid signature, ");
		if ((reason & SBX509.vrUnknownCA) == SBX509.vrUnknownCA)
			sb.append("Unknown CA, ");
		if ((reason & SBX509.vrCAUnauthorized) == SBX509.vrCAUnauthorized)
			sb.append("CA Unauthorized, ");
		if ((reason & SBX509.vrCRLNotVerified) == SBX509.vrCRLNotVerified)
			sb.append("CRL not verified, ");
		if ((reason & SBX509.vrOCSPNotVerified) == SBX509.vrOCSPNotVerified)
			sb.append("OCSP not verified, ");
		if ((reason & SBX509.vrIdentityMismatch) == SBX509.vrIdentityMismatch)
			sb.append("Identity mismatch, ");
		if ((reason & SBX509.vrNoKeyUsage) == SBX509.vrNoKeyUsage)
			sb.append("No key usage, ");
		if ((reason & SBX509.vrBlocked) == SBX509.vrBlocked)
			sb.append("Blocked, ");
		
		String res = sb.toString();
		if (res.length() > 2)
			return res.substring(0, res.length() - 2);
		else
			return res;
	}
	
	public static String xadesReasonsToString(int reasons) {
		StringBuilder sb = new StringBuilder();
		
		if ((reasons & SBXMLAdESIntf.xvrNotSigned) == SBXMLAdESIntf.xvrNotSigned)
			sb.append("Not Signed, ");
		if ((reasons & SBXMLAdESIntf.xvrNoSigningCertificate) == SBXMLAdESIntf.xvrNoSigningCertificate)
			sb.append("No Signing Certificate, ");
		if ((reasons & SBXMLAdESIntf.xvrSigningCertificateNotSigned) == SBXMLAdESIntf.xvrSigningCertificateNotSigned)
			sb.append("Signing Certificate Not Signed, ");
		if ((reasons & SBXMLAdESIntf.xvrSigningCertificateIncomplete) == SBXMLAdESIntf.xvrSigningCertificateIncomplete)
			sb.append("Signing Certificate Incomplete, ");
		if ((reasons & SBXMLAdESIntf.xvrSigningCertificateInvalid) == SBXMLAdESIntf.xvrSigningCertificateInvalid)
			sb.append("Signing Certificate Invalid, ");
		if ((reasons & SBXMLAdESIntf.xvrIndividualDataObjectsTimestampInvalid) == SBXMLAdESIntf.xvrIndividualDataObjectsTimestampInvalid)
			sb.append("Individual Data Objects Timestamp Invalid, ");
		if ((reasons & SBXMLAdESIntf.xvrIndividualDataObjectsTimestampIncomplete) == SBXMLAdESIntf.xvrIndividualDataObjectsTimestampIncomplete)
			sb.append("Individual Data Objects Timestamp Incomplete, ");
		if ((reasons & SBXMLAdESIntf.xvrAllDataObjectsTimestampInvalid) == SBXMLAdESIntf.xvrAllDataObjectsTimestampInvalid)
			sb.append("All Data Objects Timestamp Invalid, ");
		if ((reasons & SBXMLAdESIntf.xvrAllDataObjectsTimestampIncomplete) == SBXMLAdESIntf.xvrAllDataObjectsTimestampIncomplete)
			sb.append("All Data Objects Timestamp Incomplete, ");
		if ((reasons & SBXMLAdESIntf.xvrSignatureTimestampInvalid) == SBXMLAdESIntf.xvrSignatureTimestampInvalid)
			sb.append("Signature Timestamp Invalid, ");
		if ((reasons & SBXMLAdESIntf.xvrSignatureTimestampIncomplete) == SBXMLAdESIntf.xvrSignatureTimestampIncomplete)
			sb.append("Signature Timestamp Incomplete, ");
		if ((reasons & SBXMLAdESIntf.xvrRefsOnlyTimestampInvalid) == SBXMLAdESIntf.xvrRefsOnlyTimestampInvalid)
			sb.append("References Only Timestamp Invalid, ");
		if ((reasons & SBXMLAdESIntf.xvrRefsOnlyTimestampIncomplete) == SBXMLAdESIntf.xvrRefsOnlyTimestampIncomplete)
			sb.append("References Only Timestamp Incomplete, ");
		if ((reasons & SBXMLAdESIntf.xvrSigAndRefsTimestampInvalid) == SBXMLAdESIntf.xvrSigAndRefsTimestampInvalid)
			sb.append("Signature And References Timestamp Invalid, ");
		if ((reasons & SBXMLAdESIntf.xvrSigAndRefsTimestampIncomplete) == SBXMLAdESIntf.xvrSigAndRefsTimestampIncomplete)
			sb.append("Signature And References Timestamp Incomplete, ");
		if ((reasons & SBXMLAdESIntf.xvrArchiveTimestampInvalid) == SBXMLAdESIntf.xvrArchiveTimestampInvalid)
			sb.append("Archive Timestamp Invalid, ");
		if ((reasons & SBXMLAdESIntf.xvrArchiveTimestampIncomplete) == SBXMLAdESIntf.xvrArchiveTimestampIncomplete)
			sb.append("Archive Timestamp Incomplete, ");
		if ((reasons & SBXMLAdESIntf.xvrInternalError) == SBXMLAdESIntf.xvrInternalError)
			sb.append("Internal Error, ");
		
		String res = sb.toString();
		if (res.length() > 2)
			return res.substring(0, res.length() - 2);
		else
			return res;
	}
	
	public static String xadesVersionToString(int version) {
		if (version == SBXMLAdES.XAdES_v1_1_1)
			return "1.1.1";
		else if (version == SBXMLAdES.XAdES_v1_2_2)
			return "1.2.2";
		else if (version == SBXMLAdES.XAdES_v1_3_2)
			return "1.3.2";
		else if (version == SBXMLAdES.XAdES_v1_4_1)
			return "1.4.1";
		else
			return "Unknown";
	}
	
	public static String xadesFormToString(int version) {
		if (version == SBXMLAdES.XAdES)
			return "XAdES";
		else if (version == SBXMLAdES.XAdES_BES)
			return "XAdES-BES";
		else if (version == SBXMLAdES.XAdES_EPES)
			return "XAdES-EPES";
		else if (version == SBXMLAdES.XAdES_T)
			return "XAdES-T";
		else if (version == SBXMLAdES.XAdES_C)
			return "XAdES-C";
		else if (version == SBXMLAdES.XAdES_X)
			return "XAdES-X";
		else if (version == SBXMLAdES.XAdES_X_L)
			return "XAdES-X-L";
		else if (version == SBXMLAdES.XAdES_A)
			return "XAdES-A";
		else
			return "Unknown";	
	}
	
	public static String getOIDValue(TElRelativeDistinguishedName nts, byte[] s, String delim) {
		StringBuilder sb = new StringBuilder();
		String t;
		
		for (int i = 0; i < nts.getCount(); i++) {
			if (Arrays.equals(s, nts.getOID(i))) {
				t = SBUtils.stringOfBytes(nts.getValue(i));
				if (t == null || t.length() == 0)
					continue;
				
				if (sb.length() == 0) {
					sb.append(t);
					if (delim.length() == 0)
						return sb.toString();
				} else {
					sb.append(delim).append(t);
				}
			}
		}
		
		return sb.toString();
	}
}
